@@ -499,7 +499,7 @@ struct ParallelCoordinator {
     void run(size_t n, Function f, size_t nthreads) {
         // Acquire the evaluator lock to indicate that we're currently in a single
         // parallel context. This avoids wacky messages from other calls to run().
-        std::lock_guard<std::mutex> lk(coord_lock);
+        std::lock_guard<std::mutex> clk(coord_lock);
 
         // This restores the state of the unknown evaluator to what it was
         // before entry into this function, to enable nested calls to run().
@@ -521,7 +521,7 @@ struct ParallelCoordinator {
                 size_t end = std::min(n, start + jobs_per_worker);
                 if (start >= end) {
                     ncomplete++;
-                    break;
+                    continue;
                 }
 
                 // Local scope, avoid shenanigans when 'w' increments.
@@ -556,6 +556,10 @@ struct ParallelCoordinator {
                     // No throw, we need to make sure we notify the worker of (failed) completion.
                     ex.finished = true;
                     ex.error = x.what();
+                } catch (...) {
+                    // Sometimes R throws these weird Rcpp::LongjumpException errors.
+                    ex.finished = true;
+                    ex.error = "failed extraction from the unknown matrix";
                 }
 
                 // Unlock before notifying, see https://en.cppreference.com/w/cpp/thread/condition_variable
